module NetworkedRFID

  # Делегат, обрабатывающий события кардридера и преобразовывающий их в события
  # прикладывания карты.
  #
  # События прикладывания карты обрабатываются методом +card_scanned(number)+
  # делегата.
  class CardApplicationDelegate

    attr_accessor :delegate, :card_number

    state_machine :state, :initial => :waiting do

      event :rfid_read do
        transition :waiting => :card_applied
        transition :card_applied => :card_applied
      end

      event :poll_reply do
        transition :card_applied => :waiting
      end

      state :waiting
      state :card_applied

      before_transition any => :card_applied do |object, transition|
        throw :halt if object.card_number == transition.args.first.card_number
      end

      after_transition any => :card_applied do |object, transition|
        object.card_number = transition.args.first.card_number
        object.delegate.card_scanned(object.card_number)
      end

      after_transition :card_applied => :waiting do |object|
        object.card_number = nil
      end

    end

  end

end