module NetworkedRFID

  # Класс отдельного TCP/IP-ридера, отвечающий за его опрос и передачу событий
  # делегату +delegate+.
  #
  # У делегата должны быть определены методы-обработчики низкоуровневых событий
  # кард-ридера вроде +poll_reply+ и +rfid_read+.
  class RemoteRFIDReader
    POLL_PACKET = [ 0x02, 0xe0, 0x00, 0x98, 0x02,
                    0x00, 0x01, 0x7b, 0x03 ].pack('C*')

    attr_reader :ip, :port
    attr_accessor :delegate

    # Инициализирует ридер, делая его доступным по _ip_ на порту _port_.
    def initialize(ip, port)
      @ip = ip
      @port = port
      @delegate = delegate
    end

    # Отправлять запрос на опрос состояния ридера через _socket_.
    def poll(socket)
      socket.send POLL_PACKET, 0, ip, port
    end

    # Обрабатывает двоичный ответ _response_ от ридера и отправяет его
    # делегату.
    def process_response(response)
      response = RFIDReply.read(response)
      if delegate.respond_to?(response.type)
        delegate.send(response.type, response.payload)
      end
    end
  end

end