module NetworkedRFID

  # Репозиторий всех ридеров, с которыми взаимодействует система.
  class RFIDReaderRepository

    # Опросить состояние ридеров и передать каждому полученный ответ
    # на обработку.
    def poll
      readers.values.each do |reader|
        reader.poll(socket)
      end

      while have_replies?
        reply_packet = socket.recvfrom_nonblock(64)
        readers[reply_packet[1][2].to_s].process_response reply_packet[0]
      end
    end

    # Добавить ридер в репозиторий.
    def <<(reader)
      readers[reader.ip] = reader
    end

  protected

    def have_replies?
      result = IO.select([ socket ], nil, nil, 0.125)
      !result.nil? && result[0].size > 0
    end

    def socket
      @socket ||= UDPSocket.new
    end

    def readers
      @readers ||= { }
    end

  end

end